/*		Name:		serial.c
		Version:	0.0.12
		Date:		11/7/2001
		Serial interface for board or emulated board
*/
#include "serial.h"
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/poll.h>
#include "interface.h"


void (* XSV_Cleanup) (void) = NULL;

GList* serial_list_2_board=NULL;
GList* serial_list_2_computer=NULL;
uchar simwotrudata[] = {13,0,1,0,0,0,1,0,0,0,0,1,0,0,0};/*22*/

uchar simregdata[]  = {	0,2,3,4 ,1,2,3,4 ,2,2,3,4 ,3,2,3,4 ,
					0,2,3,4 ,1,2,3,4 ,2,2,3,4 ,3,2,3,4 ,
					0,2,3,4 ,1,2,3,4 ,2,2,3,4 ,3,2,3,4 ,
					0,2,3,4 ,1,2,3,4 ,2,2,3,4 ,3,2,3,4 ,
					200,255,255,255,255};

void  serial_crash(void)         /* on errors crash out here*/
{
 if (SERIAL == board_type){
	tcsetattr (serial_FD, TCSANOW, &serial_originalportsettings);
	close (serial_FD);
   	}
 exit(EXIT_FAILURE);
}

void serial_error(char *errorstring) /* on errors during setup crash here*/
{
 close (serial_FD);
 g_print ("Serial Error: %s\n", errorstring);
 exit(EXIT_FAILURE);
}


int serial_setup(void)      /* setup serial port settings*/
{
 struct termios serialsettings;
 struct sigaction cleanup;
 struct flock lock;
	XSV_Cleanup = serial_crash;
	memset ((void *) &serialsettings, 0, sizeof (struct termios));

	cleanup.sa_handler = (void (*)()) XSV_Cleanup;
	sigemptyset (&cleanup.sa_mask);
	cleanup.sa_flags = 0;	

 	sigaction (SIGINT, &cleanup, NULL); /* borrriinnnnggggggggg*/
	sigaction (SIGHUP, &cleanup, NULL);
	sigaction (SIGTERM, &cleanup, NULL);
	sigaction (SIGSEGV, &cleanup, NULL);
	sigaction (SIGBUS, &cleanup, NULL);

 serial_FD = open (PORTNAME, O_RDWR | O_NOCTTY| O_NDELAY);
 if (serial_FD < 0)
   serial_error("couldn't open serial port");
 lock.l_type = F_WRLCK;
 lock.l_whence = SEEK_SET;
 lock.l_len = 0;
 lock.l_start = 0;
 if (0 > fcntl (serial_FD, F_SETLK, &lock))
   serial_error ("failed to get serial port lock");
	fcntl (serial_FD, F_SETFL, 0);
 if (tcgetattr (serial_FD, &serial_originalportsettings) == -1)
   serial_error ("failed to set serial port attributes");
 if (cfsetispeed (&serialsettings, PORTSPEED))   /* used to be B9600 */
   serial_error ("failed to set input baud rate");
 if (cfsetospeed (&serialsettings, PORTSPEED))
   serial_error ("failed to set output baud rate");

	serialsettings.c_oflag &= ~OPOST;
	serialsettings.c_iflag |= IGNPAR;
	serialsettings.c_cflag |= CS8 | CREAD | CLOCAL;
	serialsettings.c_cc[VMIN] = 0;
	serialsettings.c_cc[VTIME] = 5;   /* <-wait time*/

 if (tcsetattr (serial_FD, TCSADRAIN, &serialsettings))
   serial_error ("failed to update serial port attributes");

  
   
 return 1;
}



int board_getchar(unsigned char *to_get) /* get 1 char from board*/
{
 return board_getchararray(1,  to_get);
}

int board_sendchar(unsigned char to_send) /*send 1 char to the board*/
{
return board_sendchararray(1, &to_send);
}


int board_getb16(int* toget) /* get 16 bit int from board*/
{
 unsigned char from_board[2];
 if (2 != board_getchararray(2, from_board))     { board_there = 0; return 0;}
 *toget=from_board[0]+(from_board[1]<<8);
 return 2;
}

int board_sendb16(int tosend) /* send 16 bit int to the board*/
{
 if (1 != board_sendchar((unsigned char) tosend & 255))
                                                 { board_there = 0; return 0;}
 if (1 != board_sendchar((unsigned char) (tosend >> 8) & 255))
                                                 { board_there = 0; return 0;}
 return 2;
}
		   
int board_sendb32(int tosend) /* get 32 bit int from board*/
{
 int x=4;
 while(x--){
   if (1 != board_sendchar((unsigned char) tosend & 255))
                                                 { board_there = 0; return 0;}
   tosend = tosend>>8;
   }
 return 4;
}

int board_getb32(int* toget) /* send 32 bit int to the board*/
{
 int x=4;
 uchar temp;
 *toget=0;
 
for (x=0;x<4;x++){
   if (1 != board_getchar(&temp))            { board_there = 0; return 0;}
   *toget= *toget | temp<<(x*8);
   }
   
 return 4;
}





int board_getchararray(int char_number, unsigned char* data_ptr)
{
 int ret=char_number; /* get char array from the board*/
 int replycount=0;
 int temp;
 struct pollfd pollfd;
 pollfd.fd=read_pipe;
 pollfd.events=POLLIN;
 if (SERIAL == board_type |
     EMULATOR == board_type ){
	while (char_number){
        if(!poll(&pollfd, 1, 100)){ if(VERBOSE|SERIAL_DEBUG) g_print("timeout2 got %d of %d\n",ret-char_number,ret);return ret-char_number;}
        replycount = read(read_pipe, data_ptr, char_number);
		if (!replycount)  { if(VERBOSE|SERIAL_DEBUG) g_print("timeout got %d of %d\n",ret-char_number,ret);return ret-char_number;}
		if (replycount<0) replycount=0;
		char_number -= replycount;
        if (SERIAL_DEBUG) while (replycount--) {g_print ("reply:%x\n",*data_ptr); data_ptr++;}
        else              data_ptr += replycount;
		}
	
	}
	
 if (FAKE == board_type){
	board_simulate();
 	while (char_number){
 		if (!serial_list_2_computer) return ret-char_number;
		*data_ptr=(uchar)(GPOINTER_TO_INT(serial_list_2_computer->data));
		data_ptr++;
		serial_list_2_computer=g_list_remove(serial_list_2_computer,serial_list_2_computer->data);
		char_number--;
		}
	}
 return ret;
}

int board_sendchararray(int char_number, unsigned char* data_ptr)
{
 struct pollfd pollfd;
 int temp=0;
 pollfd.fd=write_pipe;
 pollfd.events=POLLOUT;
 if(!poll(&pollfd, 1, 100)){ g_print("Host seems to be dead!\n");exit(1);}
 if (SERIAL_DEBUG) while(temp<char_number)  g_print ("sent:%x\n",data_ptr[temp++]);
 if (SERIAL == board_type |
     EMULATOR == board_type ) {
     write(write_pipe, data_ptr, char_number);
	 }
 
 if (FAKE   == board_type) 
 	while (temp--)
		serial_list_2_board= g_list_append(serial_list_2_board,
								GINT_TO_POINTER((gint)data_ptr[temp]));
 return char_number; /* send char array to the board*/
}




void board_simulate(void) 
{
 int temp, temp2;
 while(serial_list_2_board){
	switch (GPOINTER_TO_INT(serial_list_2_board->data))
		{
		case BR_PING:
			serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER('O'));
			serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER('K'));
			serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER('0'));
			serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER('0'));
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			break;
	   case BR_WOT_R_U:
		    for (temp=0;temp<15;temp++)
				serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER((gint)simwotrudata[temp]));
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			break;
       case BR_WOT_U_DO:
            serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
		    for (temp=0;temp<9;temp++)
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
				serial_list_2_computer= g_list_append(serial_list_2_computer,0);
            
	   case BR_NOP:
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			break;
			
	   default:
		    if (g_list_length(serial_list_2_board)<5) return;
			if ((GPOINTER_TO_UINT(serial_list_2_board->data))>>6 != 1) 	{
				serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);	
				return;
				}
			temp = (GPOINTER_TO_INT(serial_list_2_board->data));
			
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);	
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
		    temp2= GPOINTER_TO_INT(serial_list_2_board->data) +  GPOINTER_TO_INT(serial_list_2_board->next->data);
			temp2 = temp2 << (temp & 3);
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
			if (!(temp & 8)) 
				while (temp2--)	{if (temp2<69) simregdata[3-temp2]= (uchar) GPOINTER_TO_INT(serial_list_2_board->data);
								 serial_list_2_board=g_list_remove(serial_list_2_board,serial_list_2_board->data);
								}
			else
				while (temp2--)	if (temp2<69) serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER((int)simregdata[temp2]));
								else		  serial_list_2_computer= g_list_append(serial_list_2_computer,GINT_TO_POINTER(0));
			break;
		}
	}
 return;
}
